#!/usr/bin/env python

# 
#  Copyright (C) 2008  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#


# These functions provide a simple python wrapping to allow the p2h histogram
# code to function with the new client/server version of chips. Some of the 
# functions depend on the chips module having been imported into the global
# namespace, but this script does not import the chips module.

from subprocess import Popen, PIPE
from os import getpid
from pycrates import *
import numpy


def p2hDisableRedraw():
   'disables the redraw parameter for the current window if it exists'
   if info() != None: 
      set_window_redraw(0)
   return


def p2hGenerateHistogram(fname, numbins=None, low=None, high=None, binwidth=None, prop=None):
   'generate a histogram of fname using numpy.histogram to bin the data'
   d1 = TABLECrate(fname, mode='r')
   cd = d1.get_column(0);
   if low == None:
      lowv = min(cd.values)
   else:
      lowv = low
   if high == None:
      highv = max(cd.values)
   else:
      highv = high
   if (highv < lowv):
      tmp = highv
      highv = lowv
      lowv = tmp
   if binwidth != None and numbins != None:
      # error both set 
      return
   elif numbins == None and binwidth == None:
      # set bins to a default of 20 or so
      bins = 20
   elif numbins != None: 
      if numbins > 1:
         bins = numbins
      else:
         # error non-positive bin count
         return
   elif (binwidth > 0):
      # compute number of bins based on bin width
      bins = (highv - lowv) / binwidth 
      if (bins * binwidth) < (highv - lowv):
         # account for partial bins 
         bins = bins + 1 
   else:
      # error bin width too small
      return
   cd1=d1.get_column(0)
   (cnts,bin0)=numpy.histogram(cd1.values, bins, range=[lowv, highv])
   binr=bin0[1:]
   binl=bin0[0:len(bin0)-1]
   p2hDisableRedraw()
   myprops = 'window.display=false ' 
   if type(prop) == str:
      myprops = myprops + prop 
   add_histogram(binl, binr, cnts, myprops)
   set_window_redraw(1)
   return


def p2hBinHistogram(fname, bfname):
   'bins column 1 of fname using columns 1,2 of file bfname and displays the histogram'
   d1 = TABLECrate(fname, mode='r')
   d2 = TABLECrate(bfname, mode='r')
   if d2.get_ncols() < 2:
      # insufficient number of cols
      return
   cd = d1.get_column(0)
   val = cd.values
   binlen = d2.get_nrows()
   cd2 = d2.get_column(0)
   left = cd2.values
   cd2 = d2.get_column(1)
   right = d2.values
   for b in range(binlen-1):
      if (left[b] > left[b+1]) or (right[b] > right[b+1]) or (right[b] > left[b+1]):
         ordered = False  
         break;
   else:
      ordered = True

   cnts = numpy.arange(binlen)*0
   if ordered:
      # ordered w/o overlaps- use a binary search for efficiency
      for a in range(d1.get_nrows()):
         jump = binlen / 2
	 pos = jump
         while (pos >= 0):
	    jump = jump /2;
            if val[a] >= left[pos] and val[a] <= right[pos]:
               cnts[pos] = cnts[pos] + 1
               pos = -1;
            elif val[a] > right[pos]:
               pos = pos + jump + 1
               if (pos >= binlen):
                  pos = -1
            elif val[a] < left[pos]:
               pos = pos - jump - 1
            else:
               pos = -1
   else:
      # unordered or overlaps- need to check every bin- alternatively we could
      # rewrite to sort the bins and index accordingly 
      for a in range(d1.get_nrows()):
         for b in range(binlen):
            if val[a] >= left[b] and  val[a] <= right[b]:  
               cnts[b] = cnts[b] + 1
   p2hDisableRedraw()
   add_histogram(left, right, cnts, 'window.display=false')
   set_window_redraw(1)
   return  
   
  
